;; @(#) tinyigrep.el -- top level interface to igrep.el

;; @(#) $Id: tinyigrep.el,v 1.13 1997/02/27 13:49:52 jaalto NotReleased $
;; @(#) $Keywords: grep extensions $
;;
;; $KnownCompatibility: 19.28 $
;; $outlineRegexp: '@+' $
;; $bookMarkRegexp: '[-~+=*#.,'`^]+ &+\\([^ \t]+\\) [-~+=*#.,'`^]+' $
;; $PackageInstallRe: '^[ \t]*;;+[*] ' $

;; This file is not part of Emacs

;;{{{ Id

;; Copyright (C) 1996,97 Jari Aalto
;; Author:       Jari Aalto <jari.aalto@poboxes.com>
;; Maintainer:   Jari Aalto <jari.aalto@poboxes.com>
;; Created:      Dec 5 1996
;;
;; To get information on this program use ident(1) or do M-x tigr-version
;; Look at the code with folding.el, tinyfold.el, tinybm.el

;; LCD Archive Entry:
;; tinyigrep|Jari Aalto|jari.aalto@poboxes.com|
;; toplevel igrep.el, completions, user defined dir structures..|
;; 27-Feb-1997|$Revision: 1.13 $|~/misc/tinyigrep.el|

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.

;;}}}
;;{{{ Install

;; ....................................................... &t-install ...
;; Put this file on your Emacs-Lisp load path, add following into your
;; ~/.emacs startup file. Before doing require see tigr-load-hook.
;; Code can be easily ripped with tinylib.el / ti::m-pkg-rip-magic
;;
;;     (require 'tinyigrep)
;;
;;      ** YOU MUST ALSO DEFINE YOUR OWN VARIABLES **
;;      ** YOU also need igrep.el **
;;
;; Or prefer autoload: your emacs loads this package only when you
;; need it.
;;
;; This assumes that you want the package to be installed behind your
;; "\C-zg" key, if you want to relocate the menus somewhere else,
;; please make necessary modifications to tigr-load-hook and
;; tigr-:menu and to this code.
;;
;;*     ;;  first make sure the C-z prefix is available for us
;;* 	(if (not (keymapp (lookup-key global-map "\C-z")))
;;*         (global-set-key "\C-z"   nil))
;;*
;;* 	(global-set-key "\C-zg"
;;* 	  '(lambda ()
;;* 	    "This function will be replaced. It just triggers autoload."
;;* 	    (interactive)
;;* 	    ;;  This call will set up everything
;;* 	    (load "tinyigrep")
;;*
;;* 	    ;;  Now, call the real menu
;;* 	    (call-interactively (lookup-key global-map "\C-zg"))
;;* 	    ))
;;*
;;
;;
;; You should also put your customizations to separate file and add
;; (provive 'rc-tigr) to the end of the resource file.
;;
;;*   (setq tigr-load-hook '(lambda () (require 'rc-tigr "~/.emacs.tigr")))
;;*   (add-hook 'tigr-load-hook 'tigr-install-default-key-bindings)
;;
;;
;; If you have any questions, use this function
;;
;;      M-x tigr-submit-bug-report



;;}}}

;;{{{ Documentation

;; ..................................................... &t-commentary ...
;;; Commentary:

;; RESERVED prefix FOR FUNCTIONS IN THIS FILE
;; - To avoid collisions to other modules I use "tigr-" in front of
;;   every function & variable. It stands for '(ti)ny i(gr)ep'
;; - variable names contain letters 'tigr-:', excecpt version/hook/mode vars.

;;  Preface, Dec 1996
;;
;;	Some time during summer 1996 Kevin Rodgers <kevinr@ihs.com>
;;	decided to put together all grep calls to one package, igrep.el:
;;	agrep, egrep, fgrep zgrep. It also could search recursively
;;	given start trees.
;;
;;	I immediately picked up the package from the emacs sources group
;;	and started using it in my emacs. The default "grep" that comes
;;	with emacs is nothing compared to igrep.el. However, there was
;;	usually some common directories that I wanted to grep all the
;;	time, like news articles, info pages, my project directories,
;;	lisp sources, my Emacs init files. So I started to building top
;;	level interface to utilize igrep.el with predefined parameter
;;	sets.
;;
;;  Description
;;
;;	o   Toplevel igrep.el interface: Using igrep.el via easy command menu
;;	o   You can toggle igrep options while you're using the
;;	    echo-area menu.
;;	o   Predefined directory searches easily: grep all your
;;	    news files, your Emacs news files, your text files, your lisp
;;	    files, grep all manual paths... just configure one variable.
;;
;;  Do you need this package?
;;
;;	If you use Emacs "grep" then I advice you to move to igrep.el and
;;      start using this package too. It simplifies your grep tasks much
;;      more.
;;
;;	Additionally, if you have several directories where you keep some
;;	persistent data where you want to do lookup from time to time, then
;;	you propably appreciate this package. It defines all the parameters
;;	for you and all you need to supply is the SEARCH-STRING.
;;
;;  Suggestion
;;
;;	I have found usefull to keep the igrep buffer in a special frame
;;	when I work in X tube. Try using this:
;;
;;* _
;;* (if window-system
;;*    (setq special-display-buffer-names
;;*       '("*compilation*" "*grep*" "*igrep*")))
;;* _



;;}}}
;;{{{ history

;; ........................................................ &t-history ...
;;; Change Log:
;; [jari]   = <jari.aalto@poboxes.com>
;;
;; Feb	27	1997	[jari]		19.28	v1.12-1.13	NotReleased
;; - Added function tigr-fix-word which discards repeated character
;;   string like '------------' for inital prompt.
;; - New 'u' command which asks for custom user option for igrep.
;;   Options are stored into history.
;; - NEW parameter to the database variable: igrep find method for
;;   many directories and files.
;;
;; Feb	19	1997	[jari]		19.28	v1.11		NotReleased
;; - Now the last selected database is shown in the echo-area menu
;;   too (last item) so that you don}t have to guess what it was after
;;   30 minutes you used the igrep last...
;;
;; Feb	10	1997	[jari]		19.28	v1.9-1.10	NotReleased
;; - igrep.el 2.55 changed the recursive variable name. Code changed so
;;   that it is now copatible with new and old versions.
;;
;; Jan	24	1997	[jari]		19.28	v1.3-1.8	NotReleased
;; - Small changes.
;;
;; Dec	7	1996	[jari]		19.28	v1.2		NotReleased
;; - Added the menuing interface to control the parameters for igrep.
;; - Added "nil" database entry treatment. Added autoload installation
;; - Rewrote the examples.
;;
;; Dec	5	1996	[jari]		19.28	v1.1		Created



;;}}}

;;; Code:

;;{{{ setup: require

;;; ......................................................... &require ...

(require 'igrep)
(require 'tinylibm)

;;  When you load buffers from compile buffer for viewing, you really
;;  need this cache utility
;;
(require 'tinycache)
(eval-and-compile (use-dynamic-compilation))


;;}}}
;;{{{ setup: hooks

;;; ......................................................... &v-hooks ...

(defvar tigr-load-hook 'tigr-install-default-key-bindings
  "Hook that is run when package is loaded.")

;;}}}
;;{{{ setup: private

;;; ....................................................... &v-private ...
;;; Private variables


(defconst tigr-:igrep-previous-args  nil
  "List of variables used for calling igrep.")

(defvar tigr-:history-igrep nil
  "History")

(defvar tigr-:history-database nil
  "History")

(defvar tigr-:history-igrep-user-options nil
  "History")

(defvar tigr-:last-database  nil
  "Last selected database.")


;;}}}
;;{{{ setup: user config

;;; ........................................................ &v-config ...
;;; User configurable
;;;


(defvar tigr-:special-database "/"
  "*If user seelcts this database, then the current search is suggested
by looking at the buffer's current directory and file extension.

Eg. if you're in list buffer /dir1/dir2/foo.el, then the suggested
files to search are

  /dir1/dir2/*el

")

;;; ................................................. &user-copy-these ...
;;; These are examples only -- Please copy/paste to your ~/.emacs
;;; and modify to your needs.


(defvar tigr-:igrep-database nil
  "*Igrep database for group of files. The directories _must_ have
trailing slashes.

There is one special entry named tigr-:special-database which
is treated differently.

You can use following entry to tell that it should be skipped, the
DB-NAME here is \"nil\".

  '(\"nil\")

This is usefull when you build the database in a variable and you test if
certain directories exist. Like this, which builds dynamically one
entry to the tigr-:igrep-database at evaluation time.

  (list
    (if (not (file-exists-p \"/not/in/this/host\"))
       \"nil\" \"my-path-db\")
     (list
      ...go and build the correct ENTRY
      ))

Note [igrep-find fag]:

    This optional argument is very important is you grep over many
    directories and many files. It is impossible to tell in the program
    if your defined criteas generate huge listing or not.

    Defining 3rd argument as list, says that we should call igrep-find and
    not igrep function to prevent \"Arg list too long\" error. This
    variable reflects `igrep-find-use-xargs', and because nil is valid
    value, you must express it in list format.

    Valid values and their intepretation is presented below. You may gain
    performance benefit with xargs since it will invoke fewer grep
    processes. In the other hand the -exec choise gives you feeback for
    every found file as it seaches them. In xargs case you have to wait
    untill the whole list has been generated.

    '(non-nil)	find is called with ...  -print | xargs -e
    '(nil)	find is called with ...  -exec grep -n -e

Format:

    '((DB-NAME
      '(GREP-PROGRAM
	(DIR DIR ..)
	[(igrep-find flag)]
	))
      ..
      )
    ")

;;}}}
;;{{{ setup: examples

;;; ...................................................... &v-examples ...
;;  These variables show you how you set up your own igrep database


(if-t nil				;Skip over this part

  ;; Beginning of file ~/.emacs.tigr
  ;; Copy these and modify
  ;;

(defvar my-:man-path-root
  (let ((path "/usr/share/man/")	;HP-10 host
	)
    (if (file-directory-p path)
	path
      "/usr/man/"			;HP-9 host
      ))
  "*Standard Man path root. Slash required!")

(defvar my-:man-path-sections
  '("cat1.Z" "cat1m.Z"
    "cat2.Z" "cat3.Z"
    "cat4.Z" "cat5.Z"
    "cat6.Z" "cat7.Z" "cat8.Z"
    )
  "*Manual sections under my-:man-path-root.")



(defconst tigr-:igrep-database
  (list

   ;; ... ... ... ... ... ... ... ... ... ... ... ... ... ...  special ..
   ;; Make sure this entry is included.
   ;; Copy this exactly like below, you may only change
   ;; the GREP program name.
   ;;
   (list
    tigr-:special-database
    (list
     "egrep"
     (list
      nil
      )))


   ;; ... ... ... ... ... ... ... ... ... ... ... ... ... .. lisp code ..
   (list
    "lisp-emacs-distribution"
    (list
     "egrep"
     (list
      (if (file-exists-p "/opt/local/share/emacs/19.30/lisp")
	  "/opt/local/share/emacs/19.30/lisp/*el" ;hp-10
	;;
	;; I'm in HP-9 host this time
	;;
	"/mnt/elite/usr2/local/lib/emacs/19.28/lisp/*el"
	))
      ))


   ;; My 50 emacs startup files ~/.emacs.xxxx
   ;;
   (list
    "lisp-rc"
    (list
     "zgrep"
     (list
      ;; .gz and .el files
      "~/.e*"
      )))

   ;;  My primary packages are under this directory.
   ;;  These ones are installed permanently
   ;;
   (list
    "lisp-primary"
    (list
     "egrep"
     (list
      "~/elisp/*el"
      )))


   ;;  Packages I have downloaded but maybe not yet
   ;;  installed or which are under testing
   ;;
   (list
    "lisp-downloaded"
    (list
     "zgrep"
     (list
      ;; .gz and .el files, all files that I have
      ;;
      "~/elisp/*[lz]"  "~/ftp/elisp/*[lz]"
      )))


   ;;  Grep all possible lisp files everywhere
   ;;
   (list
    "load-path"
    (list
     "zgrep"
     (list-union
      (let (paths
	    )
	(mapcar
	 '(lambda (x)
	    ;;  filter out non existing paths
	    ;;
	    (if (file-exists-p x)
		(list-add
		 paths
		 (concat (ti::s-verify-ends x "/") "*[lz]")))
	    )
	 load-path)
	paths)

      (list
       "~/.emacs*" "~/ftp/elisp/*[lz]"
       ))

     '(nil)				;use -exec method and igrep-find
     ))

   ;;  Special MC tree grep,
   ;;
   (list
    ;;  Have I installed this ? (I may be working in host that doesn't
    ;;  have this)
    ;;
    (if (file-exists-p "~/elisp/pgp/mailcrypt-3.4/")
	"mc"  "nil")
    (list
     "egrep"
     (list
      "~/elisp/pgp/mailcrypt-3.4/*el"
      )))

   ;;  Special TM.el (mime) tree grep
   ;;
   (list
    (if (file-exists-p "~/elisp/mime/tm/")
	"tm"  "nil")
    (list
     "egrep"
     (list
      ;; .gz and .el files
      "~/elisp/mime/tm/*el"
      )))


   ;; ... ... ... ... ... ... ... ... ... ... ... ... ... ... . XEmacs ..
   (list
    ;;  the XE is installed into HP-10 only, I may be in HP-9
    ;;  currently...
    ;;
    (if (file-exists-p "/opt/local/lib/xemacs-19.14/")
	"xe-lisp"  "nil")
    (list
     "egrep"
     (list
      ;; symlink to /opt/local/lib/xemacs-19.14/lisp
      ;;
      "~/elisp/xe-lisp/*el"
      )))

   (list
    (if (file-exists-p "/opt/local/lib/xemacs-19.14/")
	"xe-info"  "nil")
    (list
     "egrep"
     (list
      ;;
      ;; I have made symlink from /opt/local/lib/xemacs-19.14/info/*info*
      ;; below my ~/elisp, so that I don't have to remember where it's
      ;; physically located
      ;;
      "~/elisp/xe-info/*info*"
      )))


   ;; ... ... ... ... ... ... ... ... ... ... ... ... emacs info pages ..
   (list
    "info"
    (list
     "egrep"
     (mapcar
      '(lambda (x)
	 (if (file-exists-p x)
	     (concat (ti::s-verify-ends x "/") "*")
	   ""))
      Info-default-directory-list)
     ))


   ;; ... ... ... ... ... ... ... ... ... ... ... ... . my C++ project ..

   ;;  All my work directories
   ;;
   (list
    "c++-work"
    (list
     "egrep"
     (list
      "~/work/*[ch]"
      )))

   ;;  my current projects
   ;;
   (list
    "c++-project"
    (list
     "egrep"
     (list
      "~/work/wmpman/*[ch]"
      "~/work/pmglib/*[ch]"
      )))

   ;; ... ... ... ... ... ... ... ... ... ... ... ... my C++ libraries ..
   ;; all the include directories are listed in my private
   ;; environment variable _INCDIRS
   ;;
   ;;
   (list
    "c++-include"
    (list
     "egrep"
     (list-union
      (mapcar
       '(lambda (x)
	  (setq x (ti::s-verify-ends (ti::s-sqz x) "/"))
	  (setq x (concat x "*[ch]")))
       (ti::s-get-blocks (getenv "_INCDIRS") "-i"))
      (list
       "~/work/*[ch]"
       ))))

   ;; ... ... ... ... ... ... ... ... ... ... ... ... ... .. man pages ..
   (list
    "man"
    (list
     "zgrep"
     (list-union
      ;; These are systems man paths
      ;;
      (mapcar
       '(lambda (x)
	  (format "%s%s/*" my-:man-path-root  x ))
       my-:man-path-sections)
      ;;
      ;; Add also extra man paths
      ;;
      '(
	"/usr/local/man/"
	"/usr/contrib/man/"
	))
     ))

   (list				;X pages
    "man-x"
    (list
     "zgrep"
     (list
      "/usr/bin/X11"
      )))

   ;; ... ... ... ... ... ... ... ... ... ... ...  my text directories ..
   (list
    "text"
    (list
     "zgrep"
     (list
      "~/txt/*" "~/wtxt/*"
      )))

   ;; ... ... ... ... ... ... ... ... ... ... ... ... ... my bin files ..
   (list
    "bin"
    (list
     "egrep"
     (list
      "~/bin/*" "~/bin2/*"
      )))

   ;; ... ... ... ... ... ... ... ... ... ... all stored news articles ..
   (list
    "news-all"
    (list
     "egrep"
     (list
      "~/News/*"
      )))

   ;;  Many times I remember that that particulr piece of information
   ;;  was mentioned by someone...voila! igrep helps me!
   ;;
   ;;
   (list
    "news"
    (list
     "egrep"
     (list
      "~/News/*ema*"
      )))


    ))


    (provide 'rc-tigr)
    ;;
    ;; End of file ~/.emacs.tigr

;;; --++-- --++-- --++-- --++-- --++-- --++-- --++-- --++- example end --
)

;;}}}
;;{{{ version

;;; ....................................................... &v-version ...

(defconst tigr-version-id
  "$Id: tinyigrep.el,v 1.13 1997/02/27 13:49:52 jaalto NotReleased $"
  "Latest modification time and version number.")


;;; ----------------------------------------------------------------------
;;;
(defun tigr-version (arg)
  "version information."
  (interactive "P")
   (ti::m-pkg-version-info "tinyigrep.el" arg))


;;; ----------------------------------------------------------------------
;;;
(defun tigr-submit-bug-report ()
  "Submit bug report."
  (interactive)
  (ti::m-pkg-submit-bug-report
   "tinyigrep.el"
   tigr-version-id
   '(tigr-version-id
     tigr-load-hook
     tigr-:igrep-previous-args
     tigr-:history-igrep
     tigr-:history-database
     tigr-:special-database
     tigr-:igrep-database
     )))


;;}}}

;;; ########################################################### &Funcs ###

;;{{{ Install

(defvar tigr-:menu
  '(
    (format
     "%s%sigrep: i=igrep g=guess l=last time d=dired [c=case r=recur u] %s"
     (if (eval (tigr-recursive-var))  "R " "")
     (if igrep-options (concat igrep-options " ") "")
     (if (stringp tigr-:last-database)
	 tigr-:last-database "")
     )

    (
     (?i  . ( (call-interactively 'igrep)))
     (?d  . ( (call-interactively 'dired-do-igrep)))
     (?g  . ( (call-interactively 'tigr-guess)))
     (?l  . ( (call-interactively 'tigr-as-last-time)))

     (?c  . (t
	     (progn
	       (if (string= "-i" igrep-options)
		   (setq igrep-options nil)
		 (setq igrep-options "-i")))
	     ))


     (?u  . (t
	     (let (opt
		   )
	       (setq
		opt
		(read-from-minibuffer
		 "Set igrep options: "
		 nil nil tigr-:history-igrep-user-options))

	       ;;  Should I check something here before doing the assignment?
	       (setq igrep-options opt)
	       )))

     (?r  . (t (let ((sym (tigr-recursive-var))
		     )
		 ;;  toggle value
		 (if (eval sym)
		     (set sym nil)
		   (set sym t)
		   ))))
     ))
  "
Menu for igrep.

Igrep call commands:
i  calls igrep directly
g  calls tigr-guess which let's you select database
l  calls tigr-as-last-time where you can use last database and use
   different search pattern.

Setting igrep options:
c  toggle case sensitivity option
r  toggle recursive option
u  set (u)ser options for igrep
")


;;; ----------------------------------------------------------------------
;;;
(defun tigr-install-default-key-bindings  ()
  "Install default key bindings."
  (interactive)
  (use-prefix-key global-map "\C-z")

  (global-set-key
   "\C-zg"
   (function
    (lambda (&optional arg)
      "Igrep command menu."
      (interactive "P")
      (timu-menu 'tigr-:menu arg)
      )))

  )


;;}}}
;;{{{ Igrep

;;; ----------------------------------------------------------------------
;;;
(defun tigr-recursive-var ()
  "Return igrep variable name"
  (if (boundp 'igrep-recursively)
      'igrep-recursively
    ;;  Never, 2.55
    'igrep-find
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tigr-fix-word  (word)
  "Set word to '' of it contains only repeated chars of length
5 or more."
  ;;  Be a bit nice to user; if he sits on repeated line like
  ;;  '------------------------' there is no point of
  ;;  offerering that as initial string.
  ;;
  ;;
  (if (and (> (length word) 5)
	   (string=
	    (make-string (length word)
			 (string-to-char (substring word 0 1)))
	    word))
      (setq word ""))
  word
  )


;;; ----------------------------------------------------------------------
;;;
(defun tigr-as-last-time  (pattern arg-list)
  "Call igrep like last time, with same args:
the word to be grepped and the passed args can be changed."
  (interactive
   (let* ((args    tigr-:igrep-previous-args)
	  (list    (mapcar 'prin1-to-string args))
	  (alist   (list-to-assoc-menu list))
	  (word    (tigr-fix-word (or (ti::b-read-space-word) "")))
	  dir
	  sel
	  elt
	  )


    (if (null args)
	(error "Sorry, no previus call to tinyigrep.el"))

     (setq word
	   (read-from-minibuffer
	    "Igrep pattern: "
	    (str-left word 40)
	    nil nil
	    'tigr-:history-igrep
	    ))

     (setq sel				;Previous args, allow changing
	   (completing-read
	    "select: "
	    alist
	    nil nil
	    (car list)
	    'list
	    ))

     ;;  Read possibly modified entry
     ;;
     (setq elt (read sel))
     (list word elt)
     ))

  (let* ((default-directory	default-directory)
	 (igrep-program		igrep-program) ;we may change these
	 use-find
	 files
         )

    (if (not (listpp arg-list))
	(error "No args"))

    (setq default-directory	(nth 0 arg-list)
	  igrep-program		(nth 1 arg-list)

	  ;;   2 is the pattern we don't care now
	  ;;
	  files			(nth 3 arg-list)
	  use-find		(nth 4 arg-list)
	  )
    (if (listpp use-find)
	(let ((igrep-find-use-xargs (car use-find))
	      )
	  (igrep-find igrep-program pattern files))
      (igrep igrep-program pattern files))
    ))

;;; ----------------------------------------------------------------------
;;;
(defun tigr-guess-args ()
  "Ask args to igrep. If you press RETURN when this function asks for
database, then you should give directory and file list shell regexp what
to match.

References:
  tigr-:igrep-database
"
  (let* ((table			tigr-:igrep-database)

	 ;;  Remove entries named "nil", do not offer them when
	 ;;  completing the DB name
	 ;;
	 (car-list              (delete "nil" (mapcar 'car table)))
  	 (table-completions	(list-to-assoc-menu car-list))

	 (word	 (tigr-fix-word (or (ti::b-read-space-word) "")))
	 (bfn    (or (buffer-file-name) ""))
	 (ext    (cond
		  ((string-match "\\.[ch]c?$" bfn)
		   "*[ch]")

		  ((string-match "\\.el$" bfn)
		   (concat
		    (char-to-string
		     (aref (file-name-nondirectory bfn) 0)) ;first char
		    "*el"
		    ))

		  (t
		   "*.??*"
		   )))

;;;	 (info   (strmatget "^[^-]+" 0
;;;			    (symbol-name
;;;			     (or (tib-info 'symbol) major-mode))))

	 (program-completions		;You really don't need others
	  (list-to-assoc-menu
	   '("zgrep"
	     "egrep"
	     )))
	 use-find
	 dir
	 prg
	 pattern files
	 ans
	 elt
	 ret
	 )

    (setq tigr-:last-database "nil")
    (or
     ;; ... ... ... ... ... ... ... ... ... ... ... ... ... ...  db ...
     (progn
       (setq ans (completing-read
		  "use database: "
		  table-completions
		  nil 'match
		  nil
		  'tigr-:history-database
		  ))

       ;; Did we get valid database ?
       ;;
       (if (or (var-nil-p ans)
	       (null (setq elt (nth 1 (assoc ans table)))))
	   nil
	 (setq tigr-:last-database ans)

	 (setq files	(nth 1 elt)
	       prg	(nth 0 elt)
	       use-find (and (> (length elt) 2)
			     (nth 2 elt))
	       )
	 t				;progn ret val
	 )
       )

     ;; ... ... ... ... ... ... ... ... ... ... ... ... ... ... ask ...
     (progn
       ;;  Read each directory and file to grep
       (setq files (ti::f-read-file-list))
       (if (null files)
	   (error "Need file list"))

       (setq prg
	     (completing-read
	      "Program: " program-completions nil nil "egrep"))
       ))


    (setq ret t)
    ;; ... ... ... ... ... ... ... ... ... ... ... ...  special / db . .
    (cond
     ((string= ans tigr-:special-database)
      (setq
       dir
       (read-from-minibuffer
	"Dir: "
	(if (buffer-file-name)
	    (concat
	     (file-name-directory  (buffer-file-name))
	     ext)
	  (car (ti::f-read-file-list)))
	))

      (setq files (list dir))

      (setq pattern
	    (read-from-minibuffer
	     (format "%s iGrep: " (car files))
	     word nil nil
	     'tigr-:history-igrep
	     ))
      )
     ;; ... ... ... ... ... ... ... ... ... ... ... ... ... ... rest ..
     (t
      (setq pattern (read-from-minibuffer
		     "iGrep: "
		     (str-left word 40) ;limit length
		     nil nil
		     'tigr-:history-igrep
		     ))
      (setq files (mapcar 'expand-file-name files))
      ))

;;;    (d! default-directory files)
    (list prg pattern files ret use-find)
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tigr-guess  (&optional prg pattern files do-it use-find)
  "Front-end to igrep, try to guess what directories to search
according to buffer content.

If you give empty prompt (do not select any database completion),
then you can specify all arguments.

The grep is never case sensitive.
"
  (interactive (tigr-guess-args))

   (let* ((default-directory    default-directory)
	  )
     (if-t do-it
       (setq  default-directory (or (file-name-directory (car files))
				    default-directory))
       (setcar files (file-name-nondirectory (car files)))

       (list-prepend
	tigr-:igrep-previous-args
	(list
	 default-directory
	 prg
	 pattern
	 files
	 use-find
	 ))
       (save-some-buffers 'no-ask) ;prevent stupid compile "save" asking
       (if (listpp use-find)
	   (let ((igrep-find-use-xargs (car use-find))
		 )
	     (igrep-find prg pattern files))
	 (igrep prg pattern files))
       )
     do-it
     ))


;;}}}


(provide   'tinyigrep)
(run-hooks 'tigr-load-hook)

;;; tinyigrep.el ends here
